@@ -4,7 +4,7 @@
  *
  * jws.c: functions definitions
  *
- * Copyright 2020-2021 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2020-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -35,7 +35,7 @@ static json_t * r_jws_parse_protected(const unsigned char * header_b64url) {
   unsigned char * header = NULL;
   size_t header_len = 0;
   json_t * j_return = NULL;
-  
+
   do {
     if (!o_base64url_decode(header_b64url, o_strlen((const char *)header_b64url), NULL, &header_len)) {
       y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_protected - Invalid base64");
@@ -84,7 +84,7 @@ static int r_jws_extract_header(jws_t * jws, json_t * j_header, uint32_t parse_f
 
     if (json_object_get(j_header, "jwk") != NULL && (parse_flags&R_PARSE_HEADER_JWK)) {
       r_jwk_init(&jwk);
-      if (r_jwk_import_from_json_t(jwk, json_object_get(j_header, "jwk")) == RHN_OK) {
+      if (r_jwk_import_from_json_t(jwk, json_object_get(j_header, "jwk")) == RHN_OK && r_jwk_key_type(jwk, NULL, 0)&R_KEY_TYPE_PUBLIC) {
         if (r_jwks_append_jwk(jws->jwks_pubkey, jwk) != RHN_OK) {
           ret = RHN_ERROR;
         }
@@ -94,7 +94,7 @@ static int r_jws_extract_header(jws_t * jws, json_t * j_header, uint32_t parse_f
       }
       r_jwk_free(jwk);
     }
-    
+
     if (json_object_get(j_header, "x5u") != NULL && (parse_flags&R_PARSE_HEADER_X5U)) {
       r_jwk_init(&jwk);
       if (r_jwk_import_from_x5u(jwk, x5u_flags, json_string_value(json_object_get(j_header, "x5u"))) == RHN_OK) {
@@ -229,21 +229,29 @@ static unsigned char * r_jws_sign_hmac(jws_t * jws, jwk_t * jwk) {
     alg = GNUTLS_DIG_SHA512;
   }
 
-  sig_len = gnutls_hmac_get_len(alg);
-  sig = o_malloc(sig_len);
-  sig_b64 = o_malloc(sig_len*2);
+  if (alg != GNUTLS_DIG_NULL) {
+    sig_len = gnutls_hmac_get_len(alg);
+    sig = o_malloc(sig_len);
+    sig_b64 = o_malloc(sig_len*2);
 
-  key_len = o_strlen(r_jwk_get_property_str(jwk, "k"));
-  key = o_malloc(key_len);
+    key_len = o_strlen(r_jwk_get_property_str(jwk, "k"));
+    if (key_len) {
+      key = o_malloc(key_len);
 
-  if (key != NULL) {
-    if (r_jwk_export_to_symmetric_key(jwk, key, &key_len) != RHN_OK) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error r_jwk_export_to_symmetric_key");
-      o_free(key);
-      key = NULL;
+      if (key != NULL) {
+        if (r_jwk_export_to_symmetric_key(jwk, key, &key_len) != RHN_OK) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error r_jwk_export_to_symmetric_key");
+          o_free(key);
+          key = NULL;
+        }
+      } else {
+        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error allocating resources for key");
+      }
+    } else {
+      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error key invalid, 'k' empty");
     }
   } else {
-    y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error allocating resources for key");
+    y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_sign_hmac - Error key invalid, 'alg' invalid");
   }
 
   if (key != NULL && sig != NULL && sig_b64 != NULL) {
@@ -555,23 +563,28 @@ static int r_jws_verify_sig_rsa(jws_t * jws, jwk_t * jwk, int x5u_flags) {
   }
 
   if (pubkey != NULL && GNUTLS_PK_RSA == gnutls_pubkey_get_pk_algorithm(pubkey, NULL)) {
-    sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
-    if (sig != NULL) {
-      if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
-        sig_dat.data = sig;
-        sig_dat.size = sig_len;
-        if (gnutls_pubkey_verify_data2(pubkey, alg, flag, &data, &sig_dat)) {
-          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error invalid signature");
-          ret = RHN_ERROR_INVALID;
+    if (!o_strnullempty((const char *)jws->signature_b64url)) {
+      sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
+      if (sig != NULL) {
+        if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
+          sig_dat.data = sig;
+          sig_dat.size = sig_len;
+          if (gnutls_pubkey_verify_data2(pubkey, alg, flag, &data, &sig_dat)) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error invalid signature");
+            ret = RHN_ERROR_INVALID;
+          }
+        } else {
+          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error o_base64url_decode for sig");
+          ret = RHN_ERROR;
         }
+        o_free(sig);
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error o_base64url_decode for sig");
-        ret = RHN_ERROR;
+        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error allocating resources for sig");
+        ret = RHN_ERROR_MEMORY;
       }
-      o_free(sig);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error allocating resources for sig");
-      ret = RHN_ERROR_MEMORY;
+      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Error signature empty");
+      ret = RHN_ERROR_INVALID;
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_rsa - Invalid public key");
@@ -608,51 +621,56 @@ static int r_jws_verify_sig_ecdsa(jws_t * jws, jwk_t * jwk, int x5u_flags) {
   }
 
   if (pubkey != NULL && GNUTLS_PK_EC == gnutls_pubkey_get_pk_algorithm(pubkey, NULL)) {
-    sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
-    if (sig != NULL) {
-      if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
-        if (sig_len == 64) {
-          r.size = 32;
-          r.data = sig;
-          s.size = 32;
-          s.data = sig + 32;
-        } else if (sig_len == 96) {
-          r.size = 48;
-          r.data = sig;
-          s.size = 48;
-          s.data = sig + 48;
-        } else if (sig_len == 132) {
-          r.size = 66;
-          r.data = sig;
-          s.size = 66;
-          s.data = sig + 66;
-        } else {
-          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error invalid signature length");
-          ret = RHN_ERROR_INVALID;
-        }
-
-        if (ret == RHN_OK) {
-          if (!gnutls_encode_rs_value(&sig_dat, &r, &s)) {
-            if (gnutls_pubkey_verify_data2(pubkey, alg, 0, &data, &sig_dat)) {
-              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error invalid signature");
-              ret = RHN_ERROR_INVALID;
-            }
-            if (sig_dat.data != NULL) {
-              gnutls_free(sig_dat.data);
-            }
+    if (!o_strnullempty((const char *)jws->signature_b64url)) {
+      sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
+      if (sig != NULL) {
+        if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
+          if (sig_len == 64) {
+            r.size = 32;
+            r.data = sig;
+            s.size = 32;
+            s.data = sig + 32;
+          } else if (sig_len == 96) {
+            r.size = 48;
+            r.data = sig;
+            s.size = 48;
+            s.data = sig + 48;
+          } else if (sig_len == 132) {
+            r.size = 66;
+            r.data = sig;
+            s.size = 66;
+            s.data = sig + 66;
           } else {
-            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error gnutls_encode_rs_value");
-            ret = RHN_ERROR;
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error invalid signature length");
+            ret = RHN_ERROR_INVALID;
           }
+
+          if (ret == RHN_OK) {
+            if (!gnutls_encode_rs_value(&sig_dat, &r, &s)) {
+              if (gnutls_pubkey_verify_data2(pubkey, alg, 0, &data, &sig_dat)) {
+                y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error invalid signature");
+                ret = RHN_ERROR_INVALID;
+              }
+              if (sig_dat.data != NULL) {
+                gnutls_free(sig_dat.data);
+              }
+            } else {
+              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error gnutls_encode_rs_value");
+              ret = RHN_ERROR;
+            }
+          }
+        } else {
+          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error o_base64url_decode for sig");
+          ret = RHN_ERROR;
         }
+        o_free(sig);
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error o_base64url_decode for sig");
-        ret = RHN_ERROR;
+        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error allocating resources for sig");
+        ret = RHN_ERROR_MEMORY;
       }
-      o_free(sig);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error allocating resources for sig");
-      ret = RHN_ERROR_MEMORY;
+      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Error signature empty");
+      ret = RHN_ERROR_INVALID;
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_ecdsa - Invalid public key");
@@ -681,23 +699,28 @@ static int r_jws_verify_sig_eddsa(jws_t * jws, jwk_t * jwk, int x5u_flags) {
   data.size = o_strlen((const char *)data.data);
 
   if (pubkey != NULL && GNUTLS_PK_EDDSA_ED25519 == gnutls_pubkey_get_pk_algorithm(pubkey, NULL)) {
-    sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
-    if (sig != NULL) {
-      if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
-        sig_dat.data = sig;
-        sig_dat.size = sig_len;
-        if (gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_EDDSA_ED25519, 0, &data, &sig_dat)) {
-          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error invalid signature");
-          ret = RHN_ERROR_INVALID;
+    if (!o_strnullempty((const char *)jws->signature_b64url)) {
+      sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
+      if (sig != NULL) {
+        if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
+          sig_dat.data = sig;
+          sig_dat.size = sig_len;
+          if (gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_EDDSA_ED25519, 0, &data, &sig_dat)) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error invalid signature");
+            ret = RHN_ERROR_INVALID;
+          }
+        } else {
+          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error o_base64url_decode for sig");
+          ret = RHN_ERROR;
         }
+        o_free(sig);
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error o_base64url_decode for sig");
-        ret = RHN_ERROR;
+        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error allocating resources for sig");
+        ret = RHN_ERROR_MEMORY;
       }
-      o_free(sig);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error allocating resources for sig");
-      ret = RHN_ERROR_MEMORY;
+      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Error signature empty");
+      ret = RHN_ERROR_INVALID;
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_eddsa - Invalid public key");
@@ -727,23 +750,28 @@ static int r_jws_verify_sig_es256k(jws_t * jws, jwk_t * jwk, int x5u_flags) {
   data.size = o_strlen((const char *)data.data);
 
   if (pubkey != NULL && GNUTLS_PK_EC == gnutls_pubkey_get_pk_algorithm(pubkey, NULL)) {
-    sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
-    if (sig != NULL) {
-      if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
-        sig_dat.data = sig;
-        sig_dat.size = sig_len;
-        if (gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_ECDSA_SHA256, 0, &data, &sig_dat)) {
-          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error invalid signature");
-          ret = RHN_ERROR_INVALID;
+    if (!o_strnullempty((const char *)jws->signature_b64url)) {
+      sig = o_malloc(o_strlen((const char *)jws->signature_b64url));
+      if (sig != NULL) {
+        if (o_base64url_decode(jws->signature_b64url, o_strlen((const char *)jws->signature_b64url), sig, &sig_len)) {
+          sig_dat.data = sig;
+          sig_dat.size = sig_len;
+          if (gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_ECDSA_SHA256, 0, &data, &sig_dat)) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error invalid signature");
+            ret = RHN_ERROR_INVALID;
+          }
+        } else {
+          y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error o_base64url_decode for sig");
+          ret = RHN_ERROR;
         }
+        o_free(sig);
       } else {
-        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error o_base64url_decode for sig");
-        ret = RHN_ERROR;
+        y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error allocating resources for sig");
+        ret = RHN_ERROR_MEMORY;
       }
-      o_free(sig);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error allocating resources for sig");
-      ret = RHN_ERROR_MEMORY;
+      y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Error signature empty");
+      ret = RHN_ERROR_INVALID;
     }
   } else {
     y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_verify_sig_es256k - Invalid public key");
@@ -763,7 +791,7 @@ static int r_jws_verify_sig_es256k(jws_t * jws, jwk_t * jwk, int x5u_flags) {
 
 static int _r_verify_signature(jws_t * jws, jwk_t * jwk, jwa_alg alg, int x5u_flags) {
   int ret;
-  
+
   switch (alg) {
     case R_JWA_ALG_HS256:
     case R_JWA_ALG_HS384:
@@ -1065,7 +1093,7 @@ jwa_alg r_jws_get_alg(jws_t * jws) {
 
 const char * r_jws_get_kid(jws_t * jws) {
   const char * kid = r_jws_get_header_str_value(jws, "kid");
-  if (!o_strlen(kid)) {
+  if (!!o_strnullempty(kid)) {
     if (jws != NULL && jws->token_mode == R_JSON_MODE_FLATTENED) {
       kid = json_string_value(json_object_get(json_object_get(jws->j_json_serialization, "header"), "kid"));
     }
@@ -1431,13 +1459,13 @@ int r_jws_advanced_parse(jws_t * jws, const char * jws_str, uint32_t parse_flags
 int r_jws_advanced_parsen(jws_t * jws, const char * jws_str, size_t jws_str_len, uint32_t parse_flags, int x5u_flags) {
   int ret;
   char * str = (char *)jws_str;
-  
+
   if (jws != NULL && str != NULL && jws_str_len) {
     while(isspace((unsigned char)*str) && jws_str_len) {
       str++;
       jws_str_len--;
     }
-    
+
     if (0 == o_strncmp("ey", str, 2)) {
       ret = r_jws_advanced_compact_parsen(jws, jws_str, jws_str_len, parse_flags, x5u_flags);
     } else if (*str == '{') {
@@ -1537,7 +1565,7 @@ int r_jws_advanced_compact_parsen(jws_t * jws, const char * jws_str, size_t jws_
           json_decref(jws->j_header);
 
           jws->j_header = json_incref(j_header);
-          
+
           if (!(parse_flags&R_PARSE_UNSIGNED)) {
             if (r_jws_get_alg(jws) == R_JWA_ALG_NONE) {
               y_log_message(Y_LOG_LEVEL_DEBUG, "r_jws_advanced_compact_parsen - error unsigned jws");
@@ -1559,25 +1587,30 @@ int r_jws_advanced_compact_parsen(jws_t * jws, const char * jws_str, size_t jws_
               ret = RHN_ERROR_PARAM;
               break;
             }
-            
+
             if (_r_inflate_payload(payload, payload_len, &jws->payload, &jws->payload_len) != RHN_OK) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - error _r_inflate_payload");
               ret = RHN_ERROR_PARAM;
               break;
             }
           } else {
-            o_free(jws->payload);
-            if ((jws->payload = o_malloc(payload_len+4)) == NULL) {
+            if ((payload = o_malloc(payload_len+4)) == NULL) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - error allocating resources for payload");
               ret = RHN_ERROR_MEMORY;
               break;
             }
 
-            if (!o_base64url_decode((unsigned char *)str_array[1], o_strlen(str_array[1]), jws->payload, &jws->payload_len)) {
-              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - error decoding jws->payload");
+            if (!o_base64url_decode((unsigned char *)str_array[1], o_strlen(str_array[1]), payload, &payload_len)) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - error decoding payload");
               ret = RHN_ERROR_PARAM;
               break;
             }
+
+            if (r_jws_set_payload(jws, payload, payload_len) != RHN_OK) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - Error r_jws_set_payload");
+              ret = RHN_ERROR;
+              break;
+            }
           }
 
           o_free(jws->header_b64url);
@@ -1588,6 +1621,11 @@ int r_jws_advanced_compact_parsen(jws_t * jws, const char * jws_str, size_t jws_
           if (str_array[2] != NULL) {
             jws->signature_b64url = (unsigned char *)o_strdup(str_array[2]);
           }
+          if (r_jws_get_alg(jws) != R_JWA_ALG_NONE && o_strnullempty(str_array[2])) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_advanced_compact_parsen - error invalid signature length");
+            ret = RHN_ERROR_PARAM;
+            break;
+          }
         } while (0);
         json_decref(j_header);
         o_free(str_header);
@@ -1615,11 +1653,11 @@ int r_jws_parse_json_t(jws_t * jws, json_t * jws_json, int x5u_flags) {
 int r_jws_parsen_json_str(jws_t * jws, const char * jws_json_str, size_t jws_str_len, int x5u_flags) {
   json_t * jws_json = NULL;
   int ret;
-  
+
   jws_json = json_loadb(jws_json_str, jws_str_len, JSON_DECODE_ANY, NULL);
   ret = r_jws_parse_json_t(jws, jws_json, x5u_flags);
   json_decref(jws_json);
-  
+
   return ret;
 }
 
@@ -1634,11 +1672,11 @@ int r_jws_advanced_parse_json_str(jws_t * jws, const char * jws_json_str, uint32
 int r_jws_advanced_parsen_json_str(jws_t * jws, const char * jws_json_str, size_t jws_json_str_len, uint32_t parse_flags, int x5u_flags) {
   json_t * jws_json = NULL;
   int ret;
-  
+
   jws_json = json_loadb(jws_json_str, jws_json_str_len, JSON_DECODE_ANY, NULL);
   ret = r_jws_advanced_parse_json_t(jws, jws_json, parse_flags, x5u_flags);
   json_decref(jws_json);
-  
+
   return ret;
 }
 
@@ -1647,6 +1685,7 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
   size_t header_len = 0, payload_len = 0, signature_len = 0, index = 0;
   char * str_header = NULL;
   json_t * j_header = NULL, * j_element = NULL;
+  unsigned char * payload = NULL;
 
   if (jws != NULL && json_is_object(jws_json)) {
     if (json_string_length(json_object_get(jws_json, "payload"))) {
@@ -1730,19 +1769,24 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
           jws->j_header = json_incref(j_header);
 
           // Decode payload
-          o_free(jws->payload);
-          if ((jws->payload = o_malloc(payload_len+4)) == NULL) {
+          if ((payload = o_malloc(payload_len+4)) == NULL) {
             y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error allocating resources for payload");
             ret = RHN_ERROR_MEMORY;
             break;
           }
 
-          if (!o_base64url_decode((unsigned char *)jws->payload_b64url, o_strlen((const char *)jws->payload_b64url), jws->payload, &jws->payload_len)) {
-            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error decoding jws->payload");
+          if (!o_base64url_decode((unsigned char *)jws->payload_b64url, o_strlen((const char *)jws->payload_b64url), payload, &payload_len)) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error decoding payload");
             ret = RHN_ERROR_PARAM;
             break;
           }
-          
+
+          if (r_jws_set_payload(jws, payload, payload_len) != RHN_OK) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error r_jws_set_payload");
+            ret = RHN_ERROR;
+            break;
+          }
+
           if (r_jws_extract_header(jws, json_object_get(jws_json, "header"), parse_flags, x5u_flags) != RHN_OK) {
             y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error extracting header params");
             ret = RHN_ERROR_PARAM;
@@ -1751,6 +1795,7 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
         } while (0);
         json_decref(j_header);
         o_free(str_header);
+        o_free(payload);
 
       } else {
         ret = RHN_OK;
@@ -1761,7 +1806,7 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
               ret = RHN_ERROR_PARAM;
               break;
             }
-            
+
             if (json_object_get(j_element, "header") && !json_is_object(json_object_get(j_element, "header"))) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error invalid format, the 'header property in a signature object must be a JSON object");
               ret = RHN_ERROR_PARAM;
@@ -1785,7 +1830,7 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
           y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error invalid format, signatures must be a JSON array");
           ret = RHN_ERROR_PARAM;
         }
-        
+
         if (ret == RHN_OK) {
           // Mode general - multiple signatures allowed
           jws->token_mode = R_JSON_MODE_GENERAL;
@@ -1795,7 +1840,7 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
             jws->header_b64url = NULL;
             o_free(jws->signature_b64url);
             jws->signature_b64url = NULL;
-            
+
             json_decref(jws->j_json_serialization);
             if ((jws->j_json_serialization = json_deep_copy(jws_json)) == NULL) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error setting j_json_serialization");
@@ -1817,21 +1862,27 @@ int r_jws_advanced_parse_json_t(jws_t * jws, json_t * jws_json, uint32_t parse_f
             }
 
             // Decode payload
-            o_free(jws->payload);
-            if ((jws->payload = o_malloc(payload_len+4)) == NULL) {
+            if ((payload = o_malloc(payload_len+4)) == NULL) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - error allocating resources for payload");
               ret = RHN_ERROR_MEMORY;
               break;
             }
 
-            if (!o_base64url_decode((unsigned char *)jws->payload_b64url, o_strlen((const char *)jws->payload_b64url), jws->payload, &jws->payload_len)) {
+            if (!o_base64url_decode((unsigned char *)jws->payload_b64url, o_strlen((const char *)jws->payload_b64url), payload, &payload_len)) {
               y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - error decoding jws->payload");
               ret = RHN_ERROR_PARAM;
               break;
             }
+
+            if (r_jws_set_payload(jws, payload, payload_len) != RHN_OK) {
+              y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_parse_json_t - Error r_jws_set_payload");
+              ret = RHN_ERROR;
+              break;
+            }
           } while (0);
           json_decref(j_header);
           o_free(str_header);
+          o_free(payload);
         }
 
       }
@@ -1853,7 +1904,7 @@ jws_t * r_jws_quick_parse(const char * jws_str, uint32_t parse_flags, int x5u_fl
 jws_t * r_jws_quick_parsen(const char * jws_str, size_t jws_str_len, uint32_t parse_flags, int x5u_flags) {
   jws_t * jws = NULL;
   int ret;
-  
+
   if (r_jws_init(&jws) == RHN_OK) {
     ret = r_jws_advanced_parsen(jws, jws_str, jws_str_len, parse_flags, x5u_flags);
     if (ret != RHN_OK) {
@@ -1899,7 +1950,7 @@ int r_jws_verify_signature(jws_t * jws, jwk_t * jwk_pubkey, int x5u_flags) {
           res = r_jws_extract_header(jws, j_header, R_PARSE_NONE, x5u_flags);
           json_decref(j_header);
           if (res == RHN_OK) {
-            if (o_strlen(kid)) {
+            if (!o_strnullempty(kid)) {
               if (jwk_pubkey != NULL) {
                 ret = _r_verify_signature(jws, jwk, jws->alg, x5u_flags);
               } else {
@@ -2110,8 +2161,7 @@ int r_jws_set_full_header_json_t(jws_t * jws, json_t * j_header) {
     }
     if (ret == RHN_OK) {
       json_decref(jws->j_header);
-      if ((jws->j_header = json_deep_copy(j_header)) != NULL) {
-      } else {
+      if ((jws->j_header = json_deep_copy(j_header)) == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "r_jws_set_full_header_json_t - Error setting header");
         ret = RHN_ERROR_MEMORY;
       }
@@ -2135,7 +2185,7 @@ int r_jws_set_full_header_json_str(jws_t * jws, const char * str_header) {
 
 int r_jws_set_properties(jws_t * jws, ...) {
   rhn_opt option;
-  uint ui_value;
+  unsigned int ui_value;
   int ret = RHN_OK;
   int i_value;
   rhn_int_t r_value;
@@ -2187,7 +2237,7 @@ int r_jws_set_properties(jws_t * jws, ...) {
           ret = r_jws_set_payload(jws, ustr_value, size_value);
           break;
         case RHN_OPT_SIG_ALG:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ret = r_jws_set_alg(jws, (jwa_alg)ui_value);
           break;
         case RHN_OPT_VERIFY_KEY_JWK:
@@ -2211,7 +2261,7 @@ int r_jws_set_properties(jws_t * jws, ...) {
           ret = r_jws_add_keys_json_str(jws, NULL, str_value);
           break;
         case RHN_OPT_VERIFY_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jws_add_keys_pem_der(jws, ui_value, NULL, 0, ustr_value, size_value);
@@ -2237,7 +2287,7 @@ int r_jws_set_properties(jws_t * jws, ...) {
           ret = r_jws_add_keys_json_str(jws, str_value, NULL);
           break;
         case RHN_OPT_SIGN_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jws_add_keys_pem_der(jws, ui_value, ustr_value, size_value, NULL, 0);
