@@ -4,7 +4,7 @@
  *
  * jwt.c: functions definitions
  *
- * Copyright 2020-2021 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2020-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -267,7 +267,7 @@ int r_jwt_set_full_claims_json_t(jwt_t * jwt, json_t * j_claim) {
 int r_jwt_set_full_claims_json_str(jwt_t * jwt, const char * str_claims) {
   json_t * j_claims;
   int ret;
-  if (jwt != NULL && o_strlen(str_claims)) {
+  if (jwt != NULL && !o_strnullempty(str_claims)) {
     if ((j_claims = json_loads(str_claims, JSON_DECODE_ANY, NULL)) != NULL) {
       ret = r_jwt_set_full_claims_json_t(jwt, j_claims);
       json_decref(j_claims);
@@ -1342,7 +1342,7 @@ jwt_t * r_jwt_quick_parse(const char * token, uint32_t parse_flags, int x5u_flag
 jwt_t * r_jwt_quick_parsen(const char * token, size_t token_len, uint32_t parse_flags, int x5u_flags) {
   jwt_t * jwt = NULL;
   int ret;
-  
+
   if (r_jwt_init(&jwt) == RHN_OK) {
     ret = r_jwt_advanced_parsen(jwt, token, token_len, parse_flags, x5u_flags);
     if (ret != RHN_OK) {
@@ -1718,7 +1718,7 @@ int r_jwt_verify_signature_nested(jwt_t * jwt, jwk_t * verify_key, int verify_ke
 
 int r_jwt_validate_claims(jwt_t * jwt, ...) {
   rhn_claim_opt option;
-  uint ret = RHN_OK;
+  unsigned int ret = RHN_OK;
   int i_value;
   const char * str_key, * str_value;
   json_t * j_value, * j_expected_value;
@@ -1732,48 +1732,48 @@ int r_jwt_validate_claims(jwt_t * jwt, ...) {
       switch (option) {
         case R_JWT_CLAIM_ISS:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_claim_str_value(jwt, "iss"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_claim_str_value(jwt, "iss"))) {
+            if (o_strnullempty(r_jwt_get_claim_str_value(jwt, "iss"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
           break;
         case R_JWT_CLAIM_SUB:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_claim_str_value(jwt, "sub"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_claim_str_value(jwt, "sub"))) {
+            if (o_strnullempty(r_jwt_get_claim_str_value(jwt, "sub"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
           break;
         case R_JWT_CLAIM_AUD:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_claim_str_value(jwt, "aud"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_claim_str_value(jwt, "aud"))) {
+            if (o_strnullempty(r_jwt_get_claim_str_value(jwt, "aud"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
           break;
         case R_JWT_CLAIM_JTI:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_claim_str_value(jwt, "jti"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_claim_str_value(jwt, "jti"))) {
+            if (o_strnullempty(r_jwt_get_claim_str_value(jwt, "jti"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
@@ -1864,24 +1864,24 @@ int r_jwt_validate_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_TYP:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_header_str_value(jwt, "typ"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_header_str_value(jwt, "typ"))) {
+            if (o_strnullempty(r_jwt_get_header_str_value(jwt, "typ"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
           break;
         case R_JWT_CLAIM_CTY:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             if (0 != o_strcmp(str_value, r_jwt_get_header_str_value(jwt, "cty"))) {
               ret = RHN_ERROR_PARAM;
             }
           } else {
-            if (!o_strlen(r_jwt_get_header_str_value(jwt, "cty"))) {
+            if (o_strnullempty(r_jwt_get_header_str_value(jwt, "cty"))) {
               ret = RHN_ERROR_PARAM;
             }
           }
@@ -1900,7 +1900,7 @@ int r_jwt_validate_claims(jwt_t * jwt, ...) {
 
 int r_jwt_set_claims(jwt_t * jwt, ...) {
   rhn_claim_opt option;
-  uint ret = RHN_OK;
+  unsigned int ret = RHN_OK;
   int i_value;
   const char * str_key, * str_value;
   json_t * j_value;
@@ -1914,7 +1914,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
       switch (option) {
         case R_JWT_CLAIM_ISS:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_claim_str_value(jwt, "iss", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1922,7 +1922,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_SUB:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_claim_str_value(jwt, "sub", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1930,7 +1930,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_AUD:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_claim_str_value(jwt, "aud", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1938,7 +1938,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_JTI:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_claim_str_value(jwt, "jti", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1971,7 +1971,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
         case R_JWT_CLAIM_STR:
           str_key = va_arg(vl, const char *);
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_key) && o_strlen(str_value)) {
+          if (!o_strnullempty(str_key) && !o_strnullempty(str_value)) {
             ret = r_jwt_set_claim_str_value(jwt, str_key, str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1980,7 +1980,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
         case R_JWT_CLAIM_INT:
           str_key = va_arg(vl, const char *);
           i_value = va_arg(vl, int);
-          if (o_strlen(str_key)) {
+          if (!o_strnullempty(str_key)) {
             ret = r_jwt_set_claim_int_value(jwt, str_key, i_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1989,7 +1989,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
         case R_JWT_CLAIM_JSN:
           str_key = va_arg(vl, const char *);
           j_value = va_arg(vl, json_t *);
-          if (o_strlen(str_key) && j_value != NULL) {
+          if (!o_strnullempty(str_key) && j_value != NULL) {
             ret = r_jwt_set_claim_json_t_value(jwt, str_key, j_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -1997,7 +1997,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_TYP:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_header_str_value(jwt, "typ", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -2005,7 +2005,7 @@ int r_jwt_set_claims(jwt_t * jwt, ...) {
           break;
         case R_JWT_CLAIM_CTY:
           str_value = va_arg(vl, const char *);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             ret = r_jwt_set_header_str_value(jwt, "cty", str_value);
           } else {
             ret = RHN_ERROR_PARAM;
@@ -2027,7 +2027,7 @@ int r_jwt_set_full_header_json_t(jwt_t * jwt, json_t * j_header) {
   int ret = RHN_OK;
   jwa_alg sign_alg, enc_alg;
   jwa_enc enc;
-  
+
   if (jwt != NULL && json_is_object(j_header)) {
     if (json_object_get(j_header, "alg") != NULL) {
       if ((sign_alg = r_str_to_jwa_alg(json_string_value(json_object_get(j_header, "alg")))) != R_JWA_ALG_UNKNOWN) {
@@ -2055,8 +2055,7 @@ int r_jwt_set_full_header_json_t(jwt_t * jwt, json_t * j_header) {
     }
     if (ret == RHN_OK) {
       json_decref(jwt->j_header);
-      if ((jwt->j_header = json_deep_copy(j_header)) != NULL) {
-      } else {
+      if ((jwt->j_header = json_deep_copy(j_header)) == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "r_jwt_set_full_header_json_t - Error setting header");
         ret = RHN_ERROR_MEMORY;
       }
@@ -2071,16 +2070,16 @@ int r_jwt_set_full_header_json_t(jwt_t * jwt, json_t * j_header) {
 int r_jwt_set_full_header_json_str(jwt_t * jwt, const char * str_header) {
   int ret;
   json_t * j_header = json_loads(str_header, JSON_DECODE_ANY, NULL);
-  
+
   ret = r_jwt_set_full_header_json_t(jwt, j_header);
   json_decref(j_header);
-  
+
   return ret;
 }
 
 int r_jwt_set_properties(jwt_t * jwt, ...) {
   rhn_opt option;
-  uint ui_value;
+  unsigned int ui_value;
   int ret = RHN_OK;
   int i_value;
   rhn_int_t r_value;
@@ -2155,15 +2154,15 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
           ret = r_jwt_set_full_claims_json_str(jwt, str_value);
           break;
         case RHN_OPT_ENC_ALG:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ret = r_jwt_set_enc_alg(jwt, (jwa_alg)ui_value);
           break;
         case RHN_OPT_ENC:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ret = r_jwt_set_enc(jwt, (jwa_enc)ui_value);
           break;
         case RHN_OPT_SIG_ALG:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ret = r_jwt_set_sign_alg(jwt, (jwa_alg)ui_value);
           break;
         case RHN_OPT_CIPHER_KEY:
@@ -2197,7 +2196,7 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
           ret = r_jwt_add_enc_keys_json_str(jwt, NULL, str_value);
           break;
         case RHN_OPT_ENCRYPT_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jwt_add_enc_keys_pem_der(jwt, ui_value, NULL, 0, ustr_value, size_value);
@@ -2223,7 +2222,7 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
           ret = r_jwt_add_enc_keys_json_str(jwt, str_value, NULL);
           break;
         case RHN_OPT_DECRYPT_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jwt_add_enc_keys_pem_der(jwt, ui_value, ustr_value, size_value, NULL, 0);
@@ -2249,7 +2248,7 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
           ret = r_jwt_add_sign_keys_json_str(jwt, NULL, str_value);
           break;
         case RHN_OPT_VERIFY_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jwt_add_sign_keys_pem_der(jwt, ui_value, NULL, 0, ustr_value, size_value);
@@ -2275,7 +2274,7 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
           ret = r_jwt_add_sign_keys_json_str(jwt, str_value, NULL);
           break;
         case RHN_OPT_SIGN_KEY_PEM_DER:
-          ui_value = va_arg(vl, uint);
+          ui_value = va_arg(vl, unsigned int);
           ustr_value = va_arg(vl, const unsigned char *);
           size_value = va_arg(vl, size_t);
           ret = r_jwt_add_sign_keys_pem_der(jwt, ui_value, ustr_value, size_value, NULL, 0);
@@ -2292,3 +2291,49 @@ int r_jwt_set_properties(jwt_t * jwt, ...) {
   }
   return ret;
 }
+
+int r_jwt_token_type(const char * token) {
+  return r_jwt_token_typen(token, o_strlen(token));
+}
+
+int r_jwt_token_typen(const char * token, size_t token_len) {
+  size_t nb_dots = 0, i, token_dup_len = 0;
+  int ret = R_JWT_TYPE_NONE;
+  char * token_dup = NULL, * tmp;
+
+  if (token != NULL && token_len) {
+    token_dup = o_strndup(token, token_len);
+    // Remove whitespaces and newlines
+    tmp = str_replace(token_dup, " ", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    tmp = str_replace(token_dup, "\n", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    tmp = str_replace(token_dup, "\t", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    tmp = str_replace(token_dup, "\v", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    tmp = str_replace(token_dup, "\f", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    tmp = str_replace(token_dup, "\r", "");
+    o_free(token_dup);
+    token_dup = tmp;
+    token_dup_len = o_strlen(token_dup);
+    for (i=0; i<token_dup_len; i++) {
+      if (token_dup[i] == '.') {
+        nb_dots++;
+      }
+    }
+    if (nb_dots == 2) {
+      ret = R_JWT_TYPE_SIGN;
+    } else if (nb_dots == 4) {
+      ret = R_JWT_TYPE_ENCRYPT;
+    }
+    o_free(token_dup);
+  }
+  return ret;
+}
